************************************************************************
*
* PCG: Preconditioned Conjugate Gradient Package
* Version: f77
*
************************************************************************
************************************************************************
*# 
*#                      COPYRIGHT/DISCLAIMER NOTICE
*#
*# This program was prepared by the Regents of the University of
*# California at Los Alamos National Laboratory under Contract No.
*# W-7405-ENG-36 with the U.S. Department of Energy (DOE), and by
*# the University of Texas at Austin under ARPA Contract No.
*# DABT63-92-C-0024.
*# 
*# The University of California and the University of Texas at
*# Austin have certain rights in the program pursuant to these
*# contracts.
*# 
*# Permission is hereby granted to use the program for the user's
*# own internal use.  The user is not granted the right to reproduce,
*# prepare derivative works, or redistribute the program without
*# prior permission of the University of California or the University
*# of Texas at Austin.
*# 
*# All rights in the program are reserved by the University of
*# California and the University of Texas at Austin.
*# 
*# Portions of this material resulted from work developed under a
*# U.S. Government Contract and are subject to the following
*# license: the Government is granted for itself and others acting
*# on its behalf a paid-up, nonexclusive, irrevocable worldwide
*# license in this computer software to reproduce, prepare derivative
*# works, and perform publicly and display publicly.
*# 
*# Neither the U.S. Government, the University of California nor
*# the University of Texas at Austin, nor any of their employees,
*# makes any warranty, express or implied, or assumes any liability
*# or responsibility for the use of this software.
*# 
*# Copyright (c) 1992-1995 the University of California and the
*# University of Texas at Austin.
*# 
************************************************************************
************************************************************************
************************************************************************
**  ^ROUTINE: WGMRHR - Reverse communication level routine for
**                     restarted GMRES acceleration, Householder
**                     version. (^)
**
************************************************************************
**
      subroutine zgmrhr ( ijob , ireq ,
     &       u , uexact , b ,
     &       iva , ivql , ivqr ,
     &       iwk , fwk , iparm , fparm , ier )
**
************************************************************************
**
**  ^DESCRIPTION:
**    Restarted GMRES iterative method, Householder version.  This
**    variant of GMRES gives the same iterates as the standard
**    algorithm when exact arithmetic is used, but can give a more
**    accurate solution when a high degree of accuracy is required.
**    This is done by replacing the Arnoldi Gram-Schmidt component
**    with a Householder reflection procedure.
**
**  ^AUTHOR:   wdj@beta.lanl.gov
**
**  ^MODIFIED: wdj@lithos.c3.lanl.gov Thu May  6 13:12:38 MDT 1993
**
**  ^ARGUMENTS: see *Subroutine Arguments* below.
**
**  ^REQUIREMENTS:
**    Common Blocks: none
**    Subroutines:   see below.
**
**  ^SIDE_EFFECTS:
**    See the definition of METHRDCLS to see which parameters
**    are modified
**
**  ^ALGORITHM:
**
**  ^REFERENCES:
**    Homer F. Walker, ``Implementation of the GMRES Method Using
**    Householder Transformations,'' {\sl SIAM J. Sci. Stat. Comput.},
**    vol. 9, no. 1, Jan. 1988, pp. 152--163.
**
**    H. F. Walker, ``Implementations of the GMRES Method,''
**    {\sl Computer Physics Communications, 53 (1989), pp. 311--320.
**
**  ^DOCUMENTATION:
**
**  ^MACROS:
**
************************************************************************
*
*     implicit character*1 (a-z)
*
*#    METHRAL        - (^)
      integer ijob
      integer ireq
      integer ier
      integer iwk(*)
      integer iparm(*)
      double complex       fwk(*)
      double complex       fparm(*)
      double complex u(*)
      double complex uexact(*)
      double complex b(*)
      integer ivql
      integer iva
      integer ivqr
*
        integer itimer
        double precision tc1, tr1, tc2, tr2
      integer    iitab
      integer    iftab
      integer    iwkstg
      integer    iclstg
      integer    in
      integer    is
      integer    iretlb
      integer    ivitmp, ivotmp
      integer    iv1tst, iv2tst
      integer    istab1, istab2, istab
      integer    ib1,ib2, iu1,iu2
      integer    iud
      parameter (iud  =0)
      integer    ibd
      parameter (ibd  =1)
      integer    ilbd
      parameter (ilbd =2)
      integer    irlbd
      parameter (irlbd=3)
      integer    ied
      parameter (ied  =4)
      integer    ird
      parameter (ird  =5)
      integer    ilrd
      parameter (ilrd =6)
      integer    irlrd
      parameter (irlrd=7)
      double complex      urndf
      double complex      top
      double complex      bot
      parameter (urndf=.1)
      double complex      mydot1, mydot2
      external   zargck
      external   zckstg
      external   zwrnd
      external   zifppr
      external   zitput
      external   xtimer
        integer ipme
        integer nproc
      external ximini
      external ximal
      external xifre
      external zfmini
      external zfmal
      external zffre
*     integer imv1, imv2
      integer    ir
      integer    iqlr
      integer    iqrqlr
      integer    iqlp
      integer    iv1
      integer    iv2
      integer    ipf, icos, isin, iu, iy, iyold, izc
      integer    eloc, ploc, nploc, ih, plocol, elocol, offend
      integer    ns
      integer    i, j
      integer    iunew, iuold, nm
      double complex      rdot
      double complex      rnorm
      double complex      r0norm
      double complex      vnorm
      double complex      uudot, uexdt
      double complex      vdot, ut, h, ctmp, stmp, v1, v2, denom, sum, v
     &al
*
      external   zhouse
      external   xeloc
*
*
****^^******************************************************************
*     $Modified: wdj@lanl.gov Mon Aug 28 19:20:55 MDT 1995
*     $Id: gmrhr.fm4,v 1.10 1994/11/22 05:20:22 joubert Exp $
*     $Revision: 1.0 $
*     $Log: gmrhr.fm4,v $
************************************************************************
*
      iitab = (iparm(6))
      iftab = (iparm(6))
      ir = (iparm(6))
      iqlr = (iparm(6))
      iqrqlr = (iparm(6))
      iqlp = (iparm(6))
      iv1 = (iparm(6))
      iv2 = (iparm(6))
      ipf = (iparm(6))
      icos = (iparm(6))
      isin = (iparm(6))
      iu = (iparm(6))
      iy = (iparm(6))
      iyold = (iparm(6))
      izc = (iparm(6))
      ih = (iparm(6))
*
*     *---go to the brancher unless this is the init call---
      if (ijob .ne. 1) go to 1000
*
*     *---inits----------
      iclstg = 1
      itimer = 0
      in = 0
      iparm(5) = in
      is = 0
*
*     *---initialize pointers----------
      iu1 = (iparm(6))
      iu2 = (iparm(6))
      ib1 = (iparm(6))
      ib2 = (iparm(6))
      istab1 = (iparm(6))
      istab2 = (iparm(6))
*
*     *---initialize----------
      ipme  = 0
      nproc = 1
      if (iparm(27) .eq. 0) iparm(27) = 0
      if (iparm(27) .eq. 0) then
        ier = 0
        if (ijob.eq.1 .or. ijob.eq.2 .or. ijob.eq.0) then
*       *---init integer memory----------
        call ximini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*       *---init floating point memory----------
        call zfmini ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        endif
*       *---check arguments----------
        call zargck ( iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*       *---initialize timer----------
        call xtimer (tc1,tr1,iparm(22),1)
      endif
*
*     *---start timer------
      call xtimer (tc1,tr1,iparm(22),0)
      itimer = 1
*
*     *---print i/fparm if requested----------
      if (iparm(27) .eq. 0) then
        call zifppr ( 1 , iparm, fparm, ier)
           if (ier .lt. 0) go to 900
      endif
*
*     *---reserve place to save integer and float variables----------
*     *---across revcom calls----------
      call ximal (iitab,42, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iftab,6, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
      if (iparm(2) .ge. 3) then
        write (iparm(1),'(/''     Restarted GMRES Method, Householder Ve
     &rsion.''/)') 
      endif
*
      ns = min(iparm(17),iparm(4))
*
      call zfmal (ir,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iqlp,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ier 
     &)
           if (ier .lt. 0) go to 900
      call zfmal (ih,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      if (.not.((iparm(13).eq.2).or.(iparm(13).eq.3))) then
        if (.not.((iparm(13).eq.1).or.(iparm(13).eq.3))) then
          iqlr = ir
          iqrqlr = iqlr
        else
          call zfmal (iqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iqrqlr = iqlr
        endif
      else
        if (.not.((iparm(13).eq.1).or.(iparm(13).eq.3))) then
          iqlr = ir
          call zfmal (iqrqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        else
          call zfmal (iqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          call zfmal (iqrqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        endif
      endif
      call zfmal (iv1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iv2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (ipf   ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (icos  ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (isin  ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iu    ,ns*(ns+1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (izc   ,ns+1, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iy    ,ns, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
      call zfmal (iyold ,ns, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
*
*
*     *---set u to zero----------
      if (iparm(14) .eq. -2
     &    .or. iparm(14) .eq. -1) then
          call zwfil (u,(0d0,0.0d0),
     &     iparm , fparm , iwk , fwk , ier  )
      endif
*
*     *---random u----------
      if (iparm(14) .eq. 2
     &    ) go to 5500
                 go to 5501
 5500  continue
          call zwrdot (mydot1,b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
        call zwrnd (u,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom matvec----------
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = iv1 
         ireq = 3
        iretlb = 2000
        go to 1500
 2000   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call zwrdot (mydot2,fwk(iv1),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call zwscl ( u, u, urndf*sqrt(mydot1/mydot2),  iparm , fparm ,
     & iwk , fwk , ier  )
                go to 5502
 5501 continue
 5502 continue
*
*     *---random u perturbation----------
      if (iparm(14) .eq. 3) go to 5503
                 go to 5504
 5503  continue
*        *---begin revcom matvec----------
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = iv1 
         ireq = 3
        iretlb = 2001
        go to 1500
 2001   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call zwsub ( fwk(iv1),
     &    b, fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call zwrdot (mydot1,fwk(iv1),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call zwset (fwk(iv1),u,
     &     iparm , fparm , iwk , fwk , ier  )
        call zwrnd (u,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom matvec----------
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = iv2 
         ireq = 3
        iretlb = 2002
        go to 1500
 2002   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call zwrdot (mydot2,fwk(iv2),
     &    fwk(iv2),  iparm , fparm , iwk , fwk , ier  )
          call zwsax (u,fwk(iv1),
     &    u, -urndf*sqrt(mydot1/mydot2),  iparm , fparm , iwk , fwk , ie
     &r  )
                go to 5505
 5504 continue
 5505 continue
*
*     *---begin iteration statistics calculation----------
      if (iparm(21) .eq. 1) go to 5506
                 go to 5507
 5506  continue
        call zfmal (istab1,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zfmal (istab2,8, iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        istab = istab1
        if (iparm(19) .eq. 1) then
          call zwdot (fwk(istab+iud), uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
        endif
          call zwdot (fwk(istab+ibd ), b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5509
                 go to 5510
 5509  continue
          call zwset (fwk(ib2),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5511
 5510 continue
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = ib2 
         ireq = 5
        iretlb = 2003
        go to 1500
 2003   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
 5511 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+ilbd), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call zwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5512
                 go to 5513
 5512  continue
          call zwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5514
 5513 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2004
        go to 1500
 2004   continue
         if (ijob .eq. -1) go to 900
 5514 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+irlbd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(19) .eq. 1) then
          call zwsub ( fwk(iu1),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call zwdot (fwk(istab+ied), fwk(iu1),
     &    fwk(iu1),  iparm , fparm , iwk , fwk , ier  )
        endif
*        *---begin revcom matvec----------
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ib1 
         ireq = 3
        iretlb = 2005
        go to 1500
 2005   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call zwsub ( fwk(ib1),
     &    b, fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
          call zwdot (fwk(istab+ird), fwk(ib1),
     &    fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5515
                 go to 5516
 5515  continue
          call zwset (fwk(ib2),fwk(ib1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5517
 5516 continue
          iva = ib1 
          ivql = ib2 
         ireq = 5
        iretlb = 2006
        go to 1500
 2006   continue
         if (ijob .eq. -1) go to 900
 5517 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+ilrd ), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call zwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5518
                 go to 5519
 5518  continue
          call zwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5520
 5519 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2007
        go to 1500
 2007   continue
         if (ijob .eq. -1) go to 900
 5520 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+irlrd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(2) .ge. 4) then
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        write (iparm(1),'(5x,''Initial iteration statistics'')') 
        write (iparm(1),'()') 
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+iud))
        endif
        write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ibd))
        write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ilbd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+irlbd))
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of error             '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ied))
        endif
        write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ird))
        write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ilrd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+irlrd))
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        endif
        call zffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
        call zffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
                go to 5508
 5507 continue
 5508 continue
*     *---end iteration statistics calculation----------
*
      if (((iparm(14).eq.0).or.(iparm(14).eq.-2)) .or. iparm(14).eq.-1) 
     &go to 5521
                 go to 5522
 5521  continue
          call zwset (fwk(ir),b,
     &     iparm , fparm , iwk , fwk , ier  )
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5524
                 go to 5525
 5524  continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5527
                 go to 5528
 5527  continue
          call zwset (fwk(iqlr),fwk(ir),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5529
 5528 continue
          iva = ir 
          ivql = iqlr 
         ireq = 5
        iretlb = 2008
        go to 1500
 2008   continue
         if (ijob .eq. -1) go to 900
 5529 continue
*        *---end revcom precon----------
                go to 5526
 5525 continue
 5526 continue
                go to 5523
 5522 continue
*        *---begin revcom matvec----------
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ir 
         ireq = 3
        iretlb = 2009
        go to 1500
 2009   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call zwsub ( fwk(ir),
     &    b, fwk(ir),  iparm , fparm , iwk , fwk , ier  )
        if (((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5530
                 go to 5531
 5530  continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5533
                 go to 5534
 5533  continue
          call zwset (fwk(iqlr),fwk(ir),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5535
 5534 continue
          iva = ir 
          ivql = iqlr 
         ireq = 5
        iretlb = 2010
        go to 1500
 2010   continue
         if (ijob .eq. -1) go to 900
 5535 continue
*        *---end revcom precon----------
                go to 5532
 5531 continue
 5532 continue
 5523 continue
      if (((iparm(13).eq.2).or.(iparm(13).eq.3)) .and. ((mod(iparm(15)/8
     &,2).eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8)) go to 5536
                 go to 5537
 5536  continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5539
                 go to 5540
 5539  continue
          call zwset (fwk(iqrqlr),fwk(iqlr),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5541
 5540 continue
          ivql = iqlr 
          ivqr = iqrqlr 
         ireq = 7
        iretlb = 2011
        go to 1500
 2011   continue
         if (ijob .eq. -1) go to 900
 5541 continue
*        *---end revcom precon----------
                go to 5538
 5537 continue
 5538 continue
*
          call zwrdot (rdot,fwk(iqlr),
     &    fwk(iqlr),  iparm , fparm , iwk , fwk , ier  )
      rnorm  = sqrt(abs(rdot))
      r0norm = rnorm
*
*---------------------------iteration loop------------------------------
*
 100  continue
*
        if (iparm(2) .ge. 5) then
        write (iparm(1),500) 'QL r norm           ', rnorm
        endif
*
*     *---begin user stopping test----------
        if (iparm(12).eq.-3) go to 5542
                 go to 5543
 5542  continue
          if ((mod(iparm(15)/1,2).eq.1)) then
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          iva = ir 
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          ivql = iqlr 
          endif
          if ((mod(iparm(15)/8,2).eq.1)) then
          ivqr = iqrqlr 
          endif
          ireq   = 9
        iretlb = 2012
        go to 1500
 2012   continue
          if ((mod(iparm(15)/8,2).eq.1)) then
          endif
          if ((mod(iparm(15)/4,2).eq.1)) then
          endif
          if ((mod(iparm(15)/2,2).eq.1)) then
          endif
          if ((mod(iparm(15)/1,2).eq.1)) then
          endif
          if (ier .ge. 0) then
            call zitput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          endif
          if (ijob .eq. -1) then
            if (iparm(2).ge.3 .and. iparm(27).eq.0) then
        write (iparm(1),'(/a/)') ' User-requested termination of iterati
     &ve method.'
            endif
            go to 900
          endif
                go to 5544
 5543 continue
 5544 continue
        if (iparm(12) .gt. 1) go to 5545
                 go to 5546
 5545  continue
          if (iparm(12) .eq. 1 ) then
            if (iparm(5) .eq. 0) then
              if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        '19 and 12 incompatible' )
                go to 900
              endif
          call zwdot (bot, uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
            endif
            call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , ier
     & )
           if (ier .lt. 0) go to 900
          call zwsub ( fwk(iv1tst),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call zwdot (top, fwk(iv1tst),
     &    fwk(iv1tst),  iparm , fparm , iwk , fwk , ier  )
            call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
          endif
          if (iparm(12) .eq. 2 ) then
            if (iparm(5) .eq. 0) then
          call zwdot (bot, b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
            endif
          call zwdot (top, fwk(ir),
     &    fwk(ir),  iparm , fparm , iwk , fwk , ier  )
          endif
          if (iparm(12) .eq. 3) go to 5548
                 go to 5549
 5548  continue
            if (iparm(5) .eq. 0) go to 5551
                 go to 5552
 5551  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5554
                 go to 5555
 5554  continue
          call zwdot (bot, fwk(iqlr),
     &    fwk(iqlr),  iparm , fparm , iwk , fwk , ier  )
                go to 5556
 5555 continue
                call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5557
                 go to 5558
 5557  continue
          call zwset (fwk(iv1tst),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5559
 5558 continue
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = iv1tst 
         ireq = 5
        iretlb = 2013
        go to 1500
 2013   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
 5559 continue
*        *---end revcom precon----------
          call zwdot (bot, fwk(iv1tst),
     &    fwk(iv1tst),  iparm , fparm , iwk , fwk , ier  )
                call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , f
     &wk , ier )
           if (ier .lt. 0) go to 900
 5556 continue
                go to 5553
 5552 continue
 5553 continue
          call zwdot (top, fwk(iqlr),
     &    fwk(iqlr),  iparm , fparm , iwk , fwk , ier  )
                go to 5550
 5549 continue
 5550 continue
          if (iparm(12) .eq. 4) go to 5560
                 go to 5561
 5560  continue
            if (iparm(5) .eq. 0) go to 5563
                 go to 5564
 5563  continue
              if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) go to 5566
                 go to 5567
 5566  continue
          call zwdot (bot, fwk(iqrqlr),
     &    fwk(iqrqlr),  iparm , fparm , iwk , fwk , ier  )
                go to 5568
 5567 continue
                call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
                call zfmal (iv2tst,iparm(3), iparm , fparm , iwk , fwk ,
     & ier )
           if (ier .lt. 0) go to 900
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5569
                 go to 5570
 5569  continue
          call zwset (fwk(iv1tst),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5571
 5570 continue
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          iva = ivitmp
          call zwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = iv1tst 
         ireq = 5
        iretlb = 2014
        go to 1500
 2014   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
 5571 continue
*        *---end revcom precon----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5572
                 go to 5573
 5572  continue
          call zwset (fwk(iv2tst),fwk(iv1tst),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5574
 5573 continue
          ivql = iv1tst 
          ivqr = iv2tst 
         ireq = 7
        iretlb = 2015
        go to 1500
 2015   continue
         if (ijob .eq. -1) go to 900
 5574 continue
*        *---end revcom precon----------
          call zwdot (bot, fwk(iv2tst),
     &    fwk(iv2tst),  iparm , fparm , iwk , fwk , ier  )
                call zffre (iv2tst,iparm(3)*(1), iparm , fparm , iwk , f
     &wk , ier )
           if (ier .lt. 0) go to 900
                call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , f
     &wk , ier )
           if (ier .lt. 0) go to 900
 5568 continue
                go to 5565
 5564 continue
 5565 continue
          call zwdot (top, fwk(iqrqlr),
     &    fwk(iqrqlr),  iparm , fparm , iwk , fwk , ier  )
                go to 5562
 5561 continue
 5562 continue
          if (iparm(12) .eq. 1 ) then
            if (iparm(19) .eq. 0) then
                ier = -5
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        '19 and 12 incompatible' )
                go to 900
            endif
            if (((iparm(14).eq.0).or.(iparm(14).eq.-2))) then
          call zwdot (top, uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
            else
              call zfmal (iv1tst,iparm(3), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
          call zwsub ( fwk(iv1tst),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call zwdot (top, fwk(iv1tst),
     &    fwk(iv1tst),  iparm , fparm , iwk , fwk , ier  )
              call zffre (iv1tst,iparm(3)*(1), iparm , fparm , iwk , fwk
     & , ier )
           if (ier .lt. 0) go to 900
            endif
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 6) then
          call zwdot (top, fwk(ir),
     &    fwk(ir),  iparm , fparm , iwk , fwk , ier  )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 7) then
          call zwdot (top, fwk(iqlr),
     &    fwk(iqlr),  iparm , fparm , iwk , fwk , ier  )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (iparm(12) .eq. 8) then
          call zwdot (top, fwk(iqrqlr),
     &    fwk(iqrqlr),  iparm , fparm , iwk , fwk , ier  )
            if (iparm(5) .eq. 0) bot = top
          endif
          if (dble(top) .lt. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Attempt to take sqrt of negative number' 
     &)
                go to 900
          endif
          if (dble(bot) .le. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Attempt to divide by zero' )
                go to 900
          endif
          fparm(5) = sqrt(dble(top)/dble(bot))
          call zitput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
                go to 5547
 5546 continue
 5547 continue
*     *---end user stopping test----------
*
        if (iparm(12) .eq. -1) then
          if (r0norm .eq. (0d0,0.0d0)) then
                ier = -10
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Attempt to divide by zero' )
                go to 900
          endif
          fparm(5) = rnorm/r0norm
          call zitput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
        endif
*
*       *---begin exact stopping test----------
        if (iparm(12) .eq. -2) then
          call zwsub ( fwk(iv1),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call zwrdot (uudot,fwk(iv1),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call zwrdot (uexdt,uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
          if (dble(uudot)  .lt. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Attempt to take sqrt of negative number' 
     &)
                go to 900
          endif
          if (dble(uexdt) .le. dble((0d0,0.0d0))) then
                ier = -10
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Attempt to divide by zero' )
                go to 900
          endif
          fparm(5) = sqrt(dble(uudot)/dble(uexdt))
          call zitput ( in , is ,  iparm , fparm , iwk , fwk , ier )
          if (dble(fparm(5)) .le. dble(fparm(4))) go to 2500
        endif
*       *---end exact stopping test----------
*
        if (in .ge. iparm(4)) then
                ier = 2
                call xersho ( ier, 'zgmrhr' , iparm , ' ' )
          go to 900
        endif
*
        call zckstg ( iclstg, iwkstg,  iparm , fparm , iwk , fwk , ier  
     &)
           if (ier .lt. 0) go to 900
*
*-----------------------proceed with iteration--------------------------
*
        if (is .eq. 0) then
          call zwset (fwk(iqlp+iparm(3)*((mod((is),ns+1)))),fwk(iqlr),
     &     iparm , fparm , iwk , fwk , ier  )
          ploc = 0
          eloc = 1
            nploc = iparm(3)
          call xeloc (ploc,eloc,nploc,offend, iparm , fparm , iwk , fwk 
     &, ier )
           if (ier .lt. 0) go to 900
          if (offend .eq. 0) then
            call zhouse (iqlp,is,ploc,eloc,ih,is,vnorm, iparm , fparm , 
     &iwk , fwk , ier )
          else
                ier = -7
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Problem dimension is zero' )
                go to 900
          endif
          if (vnorm .eq. (0d0,0.0d0)) then
                ier = -7
                call xersho ( ier, 'zgmrhr' , iparm , ' ' )
                go to 900
          endif
          call zwscl ( fwk(iqlp+iparm(3)*((mod((is),ns+1)))), fwk(iqlp+i
     &parm(3)*((mod((is),ns+1)))), (1d0,0.0d0)/vnorm,  iparm , fparm , i
     &wk , fwk , ier  )
          fwk((ipf  + mod((is),ns+1))) = (1d0,0.0d0)
          fwk((izc  + mod((is+1)-1,ns+1))) = vnorm
          if (iparm(2) .ge. 5) then
        write (iparm(1),500) 'h(1,0)              ', vnorm
          endif
        else
          call zwfil (fwk(iqlp+iparm(3)*((mod((is),ns+1)))),(0d0,0.0d0),
     &     iparm , fparm , iwk , fwk , ier  )
          call zwsete (fwk(iqlp+iparm(3)*((mod((is),ns+1)))),ploc,eloc,p
     &loc,eloc,(1d0,0.0d0),
     &     iparm , fparm , iwk , fwk , ier  )
          do 5575 i = is, 0, -1
          call zwdot (vdot, fwk(ih+iparm(3)*(i)),
     &    fwk(iqlp+iparm(3)*((mod((is),ns+1)))),  iparm , fparm , iwk , 
     &fwk , ier  )
          call zwsax (fwk(iqlp+iparm(3)*((mod((is),ns+1)))),fwk(iqlp+ipa
     &rm(3)*((mod((is),ns+1)))),
     &    fwk(ih+iparm(3)*(i)), -2*vdot,  iparm , fparm , iwk , fwk , ie
     &r  )
5575  continue
          fwk((ipf  + mod((is),ns+1))) = (1d0,0.0d0)
        endif
*
*
        if (offend .eq. 1) then
                ier = -6
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Number of iterations exceeds matrix dimen
     &sion' )
                go to 900
        endif
        plocol = ploc
        elocol = eloc
        eloc   = eloc + 1
        call xeloc (ploc,eloc,nploc,offend, iparm , fparm , iwk , fwk , 
     &ier )
           if (ier .lt. 0) go to 900
*
        go to (501,502,503,504), iparm(13)+1
 501    continue
*        *---begin revcom matvec----------
          ivqr = iqlp + iparm(3)*((mod((is),ns+1)))
          iva = iv1 
         ireq = 3
        iretlb = 2016
        go to 1500
 2016   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
        go to 510
 502    continue
*        *---begin revcom matvec----------
          ivqr = iqlp + iparm(3)*((mod((is),ns+1)))
          iva = iv2 
         ireq = 3
        iretlb = 2017
        go to 1500
 2017   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5576
                 go to 5577
 5576  continue
          call zwset (fwk(iv1),fwk(iv2),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5578
 5577 continue
          iva = iv2 
          ivql = iv1 
         ireq = 5
        iretlb = 2018
        go to 1500
 2018   continue
         if (ijob .eq. -1) go to 900
 5578 continue
*        *---end revcom precon----------
        go to 510
 503    continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5579
                 go to 5580
 5579  continue
          call zwset (fwk(iv2),fwk(iqlp+iparm(3)*((mod((is),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5581
 5580 continue
          ivql = iqlp + iparm(3)*((mod((is),ns+1)))
          ivqr = iv2 
         ireq = 7
        iretlb = 2019
        go to 1500
 2019   continue
         if (ijob .eq. -1) go to 900
 5581 continue
*        *---end revcom precon----------
*        *---begin revcom matvec----------
          ivqr = iv2 
          iva = iv1 
         ireq = 3
        iretlb = 2020
        go to 1500
 2020   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
        go to 510
 504    continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5582
                 go to 5583
 5582  continue
          call zwset (fwk(iv1),fwk(iqlp+iparm(3)*((mod((is),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5584
 5583 continue
          ivql = iqlp + iparm(3)*((mod((is),ns+1)))
          ivqr = iv1 
         ireq = 7
        iretlb = 2021
        go to 1500
 2021   continue
         if (ijob .eq. -1) go to 900
 5584 continue
*        *---end revcom precon----------
*        *---begin revcom matvec----------
          ivqr = iv1 
          iva = iv2 
         ireq = 3
        iretlb = 2022
        go to 1500
 2022   continue
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5585
                 go to 5586
 5585  continue
          call zwset (fwk(iv1),fwk(iv2),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5587
 5586 continue
          iva = iv2 
          ivql = iv1 
         ireq = 5
        iretlb = 2023
        go to 1500
 2023   continue
         if (ijob .eq. -1) go to 900
 5587 continue
*        *---end revcom precon----------
 510    continue
*
        do 5588 i = 0, is
          call zwdot (vdot, fwk(ih+iparm(3)*(i)),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call zwsax (fwk(iv1),fwk(iv1),
     &    fwk(ih+iparm(3)*(i)), -2*vdot,  iparm , fparm , iwk , fwk , ie
     &r  )
5588  continue
        if (offend .eq. 0) then
          call zwset (fwk(iv2),fwk(iv1),
     &     iparm , fparm , iwk , fwk , ier  )
          call zwsete (fwk(iv2),0,1,plocol,elocol,(0d0,0.0d0),
     &     iparm , fparm , iwk , fwk , ier  )
          call zhouse (iv2,0,ploc,eloc,ih,is+1,vnorm, iparm , fparm , iw
     &k , fwk , ier )
          call zwdot (vdot, fwk(ih+iparm(3)*(is+1)),
     &    fwk(iv1),  iparm , fparm , iwk , fwk , ier  )
          call zwsax (fwk(iv1),fwk(iv1),
     &    fwk(ih+iparm(3)*(is+1)), -2*vdot,  iparm , fparm , iwk , fwk ,
     & ier  )
        endif
          call zwgete (fwk(iv1),0,1,ploc,eloc,fwk((iu  + ((1)-1) + (ns+1
     &)*((is+1)-1))),
     &     iparm , fparm , iwk , fwk , ier  )
        do 5589 i = 0, is+1
          if (iparm(2) .ge. 5) then
        write (iparm(1),'(1x,a,i5,a,i5,a,1pg20.10,2x,1pg20.10)') 'h(',i+
     &1,',',is+1,')      ', fwk((iu  + ((i+1)-1) + (ns+1)*((is+1)-1)))
          endif
5589  continue
*
        iuold = (iu  + ((1)-1) + (ns+1)*((is+1)-1))
        do 5590 i = 0, is-1
          iunew = (iu  + ((i+2)-1) + (ns+1)*((is+1)-1))
          ut    = fwk(iuold)
          h     = fwk(iunew)
          ctmp  = fwk((icos  + mod((i+1),ns)))
          stmp  = fwk((isin  + mod((i+1),ns)))
          fwk(iuold) =  conjg(ctmp)*ut + conjg(stmp)*h
          fwk(iunew) =       -stmp *ut +       ctmp *h
          iuold = iunew
5590  continue
        iunew = (iu  + ((is+2)-1) + (ns+1)*((is+1)-1))
        v1 = fwk(iuold)
        v2 = fwk(iunew)
        denom = sqrt (conjg(v1)*v1 + conjg(v2)*v2)
        if (denom .eq. (0d0,0.0d0)) then
                ier = -6
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Iterative method has apparently converged
     &' )
                go to 900
        endif
        fwk((icos  + mod((is+1),ns))) = v1/denom
        fwk((isin  + mod((is+1),ns))) = v2/denom
        fwk(iuold) = denom
        fwk(iunew) = (0d0,0.0d0)
        if (iparm(2) .ge. 5) then
          do 5591 i = 1, is+1
        write (iparm(1),'(1x,a,i5,a,i5,a,1pg20.10,2x,1pg20.10)') 'u(',i,
     &',',is+1,')      ', fwk((iu  + ((i)-1) + (ns+1)*((is+1)-1)))
5591  continue
        endif
        fwk((izc  + mod((is+2)-1,ns+1))) =       -fwk((isin  + mod((is+1
     &),ns))) *fwk((izc  + mod((is+1)-1,ns+1)))
        fwk((izc  + mod((is+1)-1,ns+1))) =  conjg(fwk((icos  + mod((is+1
     &),ns))))*fwk((izc  + mod((is+1)-1,ns+1)))
*
        if (((mod(iparm(15)/1,2).eq.1)     .or.
     &    iparm(12).eq.-2    .or.
     &    iparm(12).eq.1     .or. iparm(12).eq.5) .or. ((mod(iparm(15)/2
     &,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6) .or. ((mod(iparm(15)/4
     &,2).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7) .or. ((mod(iparm(15)/8,
     &2).eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8) .or. .true.
     &            .or. (is.eq.ns-1)) then
          nm = is + 1
       do 5592 i = 1, nm-1
       fwk(iyold-1+i) = fwk(iy-1+i)
 5592  continue
          fwk(iyold-1+nm) = (0d0,0.0d0)
          do 5593 i = nm, 1, -1
            sum = fwk((izc  + mod((i)-1,ns+1)))
            do 5594 j = i+1, nm
              sum = sum - fwk(iy-1+j)*fwk((iu  + ((i)-1) + (ns+1)*((j)-1
     &)))
5594  continue
            fwk(iy-1+i) = sum/fwk((iu  + ((i)-1) + (ns+1)*((i)-1)))
5593  continue
          if (iparm(2) .ge. 5) then
            do 5595 i = 1, nm
        write (iparm(1),'(1x,a,i5,a,1pg20.10,2x,1pg20.10)') 'u-from-p(',
     &i,')     ', fwk(iy-1+i)
5595  continue
          endif
        endif
*
        if (((mod(iparm(15)/1,2).eq.1)     .or.
     &    iparm(12).eq.-2    .or.
     &    iparm(12).eq.1     .or. iparm(12).eq.5).or.((mod(iparm(15)/2,2
     &).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6).or.((mod(iparm(15)/4,2
     &).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7).or.((mod(iparm(15)/8,2)
     &.eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8).or..true.) go to 5596
                 go to 5597
 5596  continue
          i = 0
5599  continue
      if (i .gt. nm-1) go to 5600
            val = (fwk(iy+i)-fwk(iyold+i)) * fwk((ipf  + mod((i),ns+1)))
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5601
                 go to 5602
 5601  continue
          call zwset (fwk(iv2),fwk(iqlp+iparm(3)*((mod((i),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5603
 5602 continue
          ivql = iqlp + iparm(3)*((mod((i),ns+1)))
          ivqr = iv2 
         ireq = 7
        iretlb = 2024
        go to 1500
 2024   continue
         if (ijob .eq. -1) go to 900
 5603 continue
*        *---end revcom precon----------
          call zwsax (u,u,
     &    fwk(iv2), val,  iparm , fparm , iwk , fwk , ier  )
          i = i + 1
      go to 5599
5600  continue
                go to 5598
 5597 continue
          if (is .eq. ns-1) go to 5604
                 go to 5605
 5604  continue
            i = 0
5607  continue
      if (i .gt. nm-1) go to 5608
              val = fwk(iy+i) * fwk((ipf  + mod((i),ns+1)))
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5609
                 go to 5610
 5609  continue
          call zwset (fwk(iv2),fwk(iqlp+iparm(3)*((mod((i),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5611
 5610 continue
          ivql = iqlp + iparm(3)*((mod((i),ns+1)))
          ivqr = iv2 
         ireq = 7
        iretlb = 2025
        go to 1500
 2025   continue
         if (ijob .eq. -1) go to 900
 5611 continue
*        *---end revcom precon----------
          call zwsax (u,u,
     &    fwk(iv2), val,  iparm , fparm , iwk , fwk , ier  )
            i = i + 1
      go to 5607
5608  continue
                go to 5606
 5605 continue
 5606 continue
 5598 continue
*
        if (((mod(iparm(15)/2,2).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6).or.((mod(iparm(15)/4,2
     &).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7).or.((mod(iparm(15)/8,2)
     &.eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8).or..true.) go to 5612
                 go to 5613
 5612  continue
*        *---begin revcom matvec----------
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
           if (ier .lt. 0) go to 900
          ivqr = ivitmp
          call zwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ir 
         ireq = 3
        iretlb = 2026
        go to 1500
 2026   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
           if (ier .lt. 0) go to 900
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call zwsub ( fwk(ir),
     &    b, fwk(ir),  iparm , fparm , iwk , fwk , ier  )
                go to 5614
 5613 continue
 5614 continue
*
        if ((((mod(iparm(15)/4,2).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7).or.((mod(iparm(15)/8,2)
     &.eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8).or..true.).and.((iparm(
     &13).eq.1).or.(iparm(13).eq.3))) go to 5615
                 go to 5616
 5615  continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5618
                 go to 5619
 5618  continue
          call zwset (fwk(iqlr),fwk(ir),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5620
 5619 continue
          iva = ir 
          ivql = iqlr 
         ireq = 5
        iretlb = 2027
        go to 1500
 2027   continue
         if (ijob .eq. -1) go to 900
 5620 continue
*        *---end revcom precon----------
                go to 5617
 5616 continue
 5617 continue
*
        if (((mod(iparm(15)/8,2).eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8).and.((iparm(13).eq.2).o
     &r.(iparm(13).eq.3))) go to 5621
                 go to 5622
 5621  continue
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5624
                 go to 5625
 5624  continue
          call zwset (fwk(iqrqlr),fwk(iqlr),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5626
 5625 continue
          ivql = iqlr 
          ivqr = iqrqlr 
         ireq = 7
        iretlb = 2028
        go to 1500
 2028   continue
         if (ijob .eq. -1) go to 900
 5626 continue
*        *---end revcom precon----------
                go to 5623
 5622 continue
 5623 continue
*
        rdot  = conjg(fwk((izc  + mod((is+2)-1,ns+1))))*fwk((izc  + mod(
     &(is+2)-1,ns+1)))
          call zwrdot (rdot,fwk(iqlr),
     &    fwk(iqlr),  iparm , fparm , iwk , fwk , ier  )
        rnorm = sqrt(rdot)
*
        in         = in + 1
        iparm(5) = in
        is         = is + 1
        if (is .eq. ns) is = 0
*
      go to 100
*
 2500 continue
      if (iparm(2) .ge. 3) then
        write (iparm(1),'(/a,i7,a/)') '     Householder GMRES Method has
     & converged in ', iparm(5), ' iterations.'
      endif
*
*-------------------------------terminate-------------------------------
*
 900  continue
*
      if (is.ne.0 .and. .not.
     &    (((mod(iparm(15)/1,2).eq.1)     .or.
     &    iparm(12).eq.-2    .or.
     &    iparm(12).eq.1     .or. iparm(12).eq.5).or.((mod(iparm(15)/2,2
     &).eq.1)     .or.
     &    iparm(12).eq.2     .or. iparm(12).eq.6).or.((mod(iparm(15)/4,2
     &).eq.1)   .or.
     &    iparm(12).eq.3    .or. iparm(12).eq.7).or.((mod(iparm(15)/8,2)
     &.eq.1) .or.
     &    iparm(12).eq.4    .or. iparm(12).eq.8).or..true.)) go to 5627
                 go to 5628
 5627  continue
        nm = is
        do 5630 i = nm, 1, -1
          sum = fwk((izc  + mod((i)-1,ns+1)))
          do 5631 j = i+1, nm
            sum = sum - fwk(iy-1+j)*fwk((iu  + ((i)-1) + (ns+1)*((j)-1))
     &)
5631  continue
          fwk(iy-1+i) = sum/fwk((iu  + ((i)-1) + (ns+1)*((i)-1)))
5630  continue
        if (iparm(2) .ge. 5) then
          do 5632 i = 1, nm
        write (iparm(1),'(1x,a,i5,a,1pg20.10,2x,1pg20.10)') 'u-from-p(',
     &i,')     ', fwk(iy-1+i)
5632  continue
        endif
        i = 0
5633  continue
      if (i .gt. nm-1) go to 5634
          val = fwk(iy+i) * fwk((ipf  + mod((i),ns+1)))
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5635
                 go to 5636
 5635  continue
          call zwset (fwk(iv2),fwk(iqlp+iparm(3)*((mod((i),ns+1)))),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5637
 5636 continue
          ivql = iqlp + iparm(3)*((mod((i),ns+1)))
          ivqr = iv2 
         ireq = 7
        iretlb = 2029
        go to 1500
 2029   continue
         if (ijob .eq. -1) go to 900
 5637 continue
*        *---end revcom precon----------
          call zwsax (u,u,
     &    fwk(iv2), val,  iparm , fparm , iwk , fwk , ier  )
        i = i + 1
      go to 5633
5634  continue
                go to 5629
 5628 continue
 5629 continue
*
      if (iclstg .ne. 1) then
        call zckstg (-1, iwkstg,  iparm , fparm , iwk , fwk , ier )
      endif
*
*     *---begin iteration statistics calculation----------
      if (iparm(21).eq.1 .and. ier.ge.0) go to 5638
                 go to 5639
 5638  continue
        call zfmal (ib1,iparm(3), iparm , fparm , iwk , fwk , ier )
        call zfmal (ib2,iparm(3), iparm , fparm , iwk , fwk , ier )
        call zfmal (iu1,iparm(3), iparm , fparm , iwk , fwk , ier )
        call zfmal (iu2,iparm(3), iparm , fparm , iwk , fwk , ier )
        istab = istab2
        if (iparm(19) .eq. 1) then
          call zwdot (fwk(istab+iud), uexact,
     &    uexact,  iparm , fparm , iwk , fwk , ier  )
        endif
          call zwdot (fwk(istab+ibd ), b,
     &    b,  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5641
                 go to 5642
 5641  continue
          call zwset (fwk(ib2),b,
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5643
 5642 continue
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          iva = ivitmp
          call zwset (fwk(iva),b,
     &     iparm , fparm , iwk , fwk , ier  )
          ivql = ib2 
         ireq = 5
        iretlb = 2030
        go to 1500
 2030   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
         if (ijob .eq. -1) go to 900
 5643 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+ilbd), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call zwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5644
                 go to 5645
 5644  continue
          call zwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5646
 5645 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2031
        go to 1500
 2031   continue
         if (ijob .eq. -1) go to 900
 5646 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+irlbd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(19) .eq. 1) then
          call zwsub ( fwk(iu1),
     &    u, uexact,  iparm , fparm , iwk , fwk , ier  )
          call zwdot (fwk(istab+ied), fwk(iu1),
     &    fwk(iu1),  iparm , fparm , iwk , fwk , ier  )
        endif
*        *---begin revcom matvec----------
          call zfmal (ivitmp,iparm(3), iparm , fparm , iwk , fwk , ier )
          ivqr = ivitmp
          call zwset (fwk(ivqr),u,
     &     iparm , fparm , iwk , fwk , ier  )
          iva = ib1 
         ireq = 3
        iretlb = 2032
        go to 1500
 2032   continue
          call zffre (ivitmp,iparm(3)*(1), iparm , fparm , iwk , fwk , i
     &er )
         if (ijob .eq. -1) go to 900
*        *---end revcom matvec----------
          call zwsub ( fwk(ib1),
     &    b, fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
          call zwdot (fwk(istab+ird), fwk(ib1),
     &    fwk(ib1),  iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.1).or.(iparm(13).eq.3))) go to 5647
                 go to 5648
 5647  continue
          call zwset (fwk(ib2),fwk(ib1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5649
 5648 continue
          iva = ib1 
          ivql = ib2 
         ireq = 5
        iretlb = 2033
        go to 1500
 2033   continue
         if (ijob .eq. -1) go to 900
 5649 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+ilrd ), fwk(ib2),
     &    fwk(ib2),  iparm , fparm , iwk , fwk , ier  )
          call zwset (fwk(iu1),fwk(ib2),
     &     iparm , fparm , iwk , fwk , ier  )
*        *---begin revcom precon----------
         if (.not. ((iparm(13).eq.2).or.(iparm(13).eq.3))) go to 5650
                 go to 5651
 5650  continue
          call zwset (fwk(iu2),fwk(iu1),
     &     iparm , fparm , iwk , fwk , ier  )
                go to 5652
 5651 continue
          ivql = iu1 
          ivqr = iu2 
         ireq = 7
        iretlb = 2034
        go to 1500
 2034   continue
         if (ijob .eq. -1) go to 900
 5652 continue
*        *---end revcom precon----------
          call zwdot (fwk(istab+irlrd), fwk(iu2),
     &    fwk(iu2),  iparm , fparm , iwk , fwk , ier  )
        if (iparm(19) .eq. 1) then
          if (fwk(istab1+ied) .eq. (0d0,0.0d0)) then
                ier = 4
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Value of RELERR may be inaccurate' )
          else
            fparm(8) = sqrt(fwk(istab2+ied)/fwk(istab1+ied))
          endif
        endif
        if (fwk(istab1+ird) .eq. (0d0,0.0d0)) then
                ier = 4
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Value of RELRSD may be inaccurate' )
        else
          fparm(7) = sqrt(fwk(istab2+ird)/fwk(istab1+ird))
        endif
        if (iparm(2) .ge. 4) then
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        write (iparm(1),'(5x,''Final iteration statistics'')') 
        write (iparm(1),'()') 
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of uexact            '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+iud))
        endif
        write (iparm(1),'(5x,''2-norm of b                 '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ibd))
        write (iparm(1),'(5x,''2-norm of Ql*b              '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ilbd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*b           '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+irlbd))
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''2-norm of error             '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ied))
        endif
        write (iparm(1),'(5x,''2-norm of residual          '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ird))
        write (iparm(1),'(5x,''2-norm of Ql*r              '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+ilrd))
        write (iparm(1),'(5x,''2-norm of Qr*Ql*r           '',1pg20.10,2
     &x,1pg20.10)') sqrt(fwk(istab+irlrd))
        if (iparm(19) .eq. 1) then
        write (iparm(1),'(5x,''Relative 2-norm of error    '',1pg20.10,2
     &x,1pg20.10)') fparm(8)
        endif
        write (iparm(1),'(5x,''Relative 2-norm of residual '',1pg20.10,2
     &x,1pg20.10)') fparm(7)
        write (iparm(1),'()') 
        write (iparm(1),'(1x, 60(''*''))') 
        write (iparm(1),'()') 
        endif
        call zffre (iu2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call zffre (iu1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call zffre (ib2,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call zffre (ib1,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
        call zffre (istab2,8, iparm , fparm , iwk , fwk , ier )
        call zffre (istab1,8, iparm , fparm , iwk , fwk , ier )
                go to 5640
 5639 continue
 5640 continue
*     *---end iteration statistics calculation----------
*
      call zffre (iyold ,ns, iparm , fparm , iwk , fwk , ier )
      call zffre (iy    ,ns, iparm , fparm , iwk , fwk , ier )
      call zffre (izc   ,ns+1, iparm , fparm , iwk , fwk , ier )
      call zffre (iu    ,ns*(ns+1), iparm , fparm , iwk , fwk , ier )
      call zffre (isin  ,ns+1, iparm , fparm , iwk , fwk , ier )
      call zffre (icos  ,ns+1, iparm , fparm , iwk , fwk , ier )
      call zffre (ipf   ,ns+1, iparm , fparm , iwk , fwk , ier )
      call zffre (iv2 ,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      call zffre (iv1 ,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
      if (.not.((iparm(13).eq.1).or.(iparm(13).eq.3))) then
        if (.not.((iparm(13).eq.2).or.(iparm(13).eq.3))) then
        else
          call zfmal (iqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
        endif
      else
        if (.not.((iparm(13).eq.2).or.(iparm(13).eq.3))) then
          call zfmal (iqrqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
        else
          call zfmal (iqrqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
          call zfmal (iqlr,iparm(3), iparm , fparm , iwk , fwk , ier )
        endif
      endif
      call zffre (ih,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ier )
      call zffre (iqlp,iparm(3)*(ns+1), iparm , fparm , iwk , fwk , ier 
     &)
      call zffre (ir,iparm(3)*(1), iparm , fparm , iwk , fwk , ier )
*
*
*
*
      call zffre (iftab,6, iparm , fparm , iwk , fwk , ier )
      call xifre (iitab,42, iparm , fparm , iwk , fwk , ier )
*
      if (itimer .eq. 1) then
        call xtimer (tc2,tr2,iparm(22),0)
        fparm(1) = fparm(1) + (tc2-tc1)
        fparm(2) = fparm(2) + (tr2-tr1)
      endif
*
 910  continue
      if (iparm(27) .eq. 0) then
        call zifppr ( -1 , iparm, fparm, ier )
      endif
*
*     *---reset iclev----------
      if (iparm(27) .eq. 0) iparm(27) = 0
      iparm(11) = (iparm(6))
      ireq        = -1
      return
*
*-------------------------------brancher--------------------------------
*
*
 1000 continue
*
      ipme  = 0
      nproc = 1
      iitab = iparm(11)
      if (iitab.eq.(iparm(6)) .and. ijob.eq.-1) go to 910
*
      if (iparm(11) .eq. (iparm(6))) then
                ier = -4
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Values of ijob and iparm(iptr) incompatib
     &le' )
      endif
*
      call xtimer (tc1,tr1,iparm(22),0)
*
*     *---remember parameters out of tables----------
        iitab  = iwk(iitab+0)
        iftab  = iwk(iitab+1)
        istab  = iwk(iitab+2)
        iwkstg = iwk(iitab+3)
        iclstg = iwk(iitab+4)
        in     = iwk(iitab+5)
        is     = iwk(iitab+6)
        itimer = iwk(iitab+7)
        iretlb = iwk(iitab+8)
        ivitmp = iwk(iitab+9)
        ivotmp = iwk(iitab+10)
        iv1tst = iwk(iitab+11)
        iv2tst = iwk(iitab+12)
        istab1 = iwk(iitab+13)
        istab2 = iwk(iitab+14)
        ib1    = iwk(iitab+15)
        ib2    = iwk(iitab+16)
        iu1    = iwk(iitab+17)
        iu2    = iwk(iitab+18)
        ir     = iwk(iitab+19)
        iqlr   = iwk(iitab+20)
        iqrqlr = iwk(iitab+21)
        iqlp   = iwk(iitab+22)
        iv1    = iwk(iitab+23)
        iv2    = iwk(iitab+24)
        ipf    = iwk(iitab+25)
        icos   = iwk(iitab+26)
        isin  = iwk(iitab+27)
        iu     = iwk(iitab+28)
        iy     = iwk(iitab+29)
        iyold  = iwk(iitab+30)
        izc   = iwk(iitab+31)
        eloc   = iwk(iitab+32)
        ploc   = iwk(iitab+33)
        nploc = iwk(iitab+34)
        ih     = iwk(iitab+35)
        plocol = iwk(iitab+36)
        elocol = iwk(iitab+37)
        ns     = iwk(iitab+38)
        i      = iwk(iitab+39)
        j     = iwk(iitab+40)
        offend = iwk(iitab+41)
        top = fwk(iftab+0)
        bot = fwk(iftab+1)
        rdot   = fwk(iftab+2)
        rnorm  = fwk(iftab+3)
        r0norm = fwk(iftab+4)
        vnorm = fwk(iftab+5)
*
      if (ijob.ne.3 .and. ijob.ne.-1) then
                ier = -4
                call xersho ( ier, 'zgmrhr' , iparm ,
     &                        'Argument ijob invalid' )
      endif
*
*     *---perform the branch to where we left off----------
      go to (2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,2011
     &,2012,2013,2014,2015,2016,2017,2018,2019,2020,2021,2022,2023,2024,
     &2025,2026,2027,2028,2029,2030,2031,2032,2033,2034),
     &       iretlb - 2000 + 1
*
*---------------------------request of caller---------------------------
*
*
 1500 continue
      iparm(11) = iitab
*     *---save tables----------
        iwk(iitab+0) = iitab 
        iwk(iitab+1) = iftab 
        iwk(iitab+2) = istab 
        iwk(iitab+3) = iwkstg
        iwk(iitab+4) = iclstg
        iwk(iitab+5) = in    
        iwk(iitab+6) = is    
        iwk(iitab+7) = itimer
        iwk(iitab+8) = iretlb
        iwk(iitab+9) = ivitmp
        iwk(iitab+10) = ivotmp
        iwk(iitab+11) = iv1tst
        iwk(iitab+12) = iv2tst
        iwk(iitab+13) = istab1
        iwk(iitab+14) = istab2
        iwk(iitab+15) = ib1   
        iwk(iitab+16) = ib2   
        iwk(iitab+17) = iu1   
        iwk(iitab+18) = iu2   
        iwk(iitab+19) = ir    
        iwk(iitab+20) = iqlr  
        iwk(iitab+21) = iqrqlr
        iwk(iitab+22) = iqlp  
        iwk(iitab+23) = iv1   
        iwk(iitab+24) = iv2   
        iwk(iitab+25) = ipf   
        iwk(iitab+26) = icos  
        iwk(iitab+27) = isin 
        iwk(iitab+28) = iu    
        iwk(iitab+29) = iy    
        iwk(iitab+30) = iyold 
        iwk(iitab+31) = izc  
        iwk(iitab+32) = eloc  
        iwk(iitab+33) = ploc  
        iwk(iitab+34) = nploc
        iwk(iitab+35) = ih    
        iwk(iitab+36) = plocol
        iwk(iitab+37) = elocol
        iwk(iitab+38) = ns    
        iwk(iitab+39) = i     
        iwk(iitab+40) = j    
        iwk(iitab+41) = offend
        fwk(iftab+0) = top
        fwk(iftab+1) = bot
        fwk(iftab+2) = rdot  
        fwk(iftab+3) = rnorm 
        fwk(iftab+4) = r0norm
        fwk(iftab+5) = vnorm
      call xtimer (tc2,tr2,iparm(22),0)
      fparm(1) = fparm(1) + (tc2-tc1)
      fparm(2) = fparm(2) + (tr2-tr1)
      return
*
*
*--------------------------------end------------------------------------
*
  500 format ( 1x, a, 1pg20.10,2x,1pg20.10 )
*
      end
*-----------------------------------------------------------------------
